[![logo](https://www.hongsnet.net/images/logo.gif)](https://www.hongsnet.net)

# Issue

- OO 서비스의 빈번한 일시적 접속 지연 발생에 따른 **Risk Management 상황** 발생
  - 개발 부서 의 분석결과, **Page Cached 영역** 사용 부족에 따른 긴급 증설 요청 의견
    - 접속 지연 이상징후 발생 시 프로세스 재기동 수행 후 해소 (근본원인파악 X)
  - 운용 부서 의 분석결과, 메모리는 30% 정도 여유가 있는 상태이고, SWAP 은 어플리케이션의 특성이라는 의견
    - SWAP 영역의 지속적 상승 유지에 따른 서비스 접속지연 발생의 근본 원인 분석
- <u>개발 / 운용 부서와의 의견충돌이 발생</u> 되고 있으며, 이를 객관화하기 위한 `세부 의미있는 데이터`는 미존재
  - 아래 그래프 최근 3 개월 기준으로는 개발 부서에서 메모리 용량부족 의견에 부합되지 않음

![issue](./images/04p_23-02-16_custom.png)


# Analysis

- OO 서비스는 **Linux Page Cache 영역을 공유 메모리로 사용**
- Linux Kernel은 메모리 가용량에 대해 Page Cache 로 활용하려는 메커니즘
- 실질 시스템 메모리 가용량이 있는 것인지, 부족한 것인지에 대한 일반적인 기술 Based 접근 시 충돌 불가피
  - Linux 는 메모리 가용량에 대해 Page Cache 로 활용“, “OO 서비스 프로세스는 Pag e Cache 일부를 사용
- OO 서비스의 시스템 메모리 `상세 현황`이 없는 상태 (**신속한 모니터링 아이템 발굴 필요**)
  - 분석 결과에 대한 객관성 타당성 부족


# Solution

- **시스템 메모리** 사용 현황의 객관성 및 타당성을 위한 **모니터링 아이템** 발굴 및 긴급 적용 진행 (총 12 대)
- 아이템 발굴 내역
  - Page Cache, Free Memory(Allocable immediately), Avail Memory(free + buffers + cached)
  - Mapped, Dirty, Active(anon, file), Inactive(anon, file), Writeback , OO 서비스 프로세스 공유메모리 사용량
  - Slab, KernelStack , Reclaim
  - Swapping, SwapCache
- 신규 모니터링 아이템 발굴에 따른 `운용 조직 내 수평 전파` 필요 (완료)

# Result

- **개발 운용 부서의 Linux 시스템 기반 서비스 에 대한 메모리 사용의 구조 파악에 일조**
   - **Page Cache** 영역의 사용은 `mmap 시스템콜에 따른 파일 매핑 영역` 메모리 증설이 일부 필요한 이유가 검증됨
   - **Active(anon)** 메모리 영역은 줄어들지 않고, 상승만 되는 구조 (Memory Leak 의심되며, 개발부서 인지됨)
   - Swap 영역 의 지속적 상승 이유는 메모리 증설필요와 연관되며, Memory Leak 해결 시 서비스 안정성 강화 기대
- 기술 일반적 관점에서 이를 `객관화 사실화 하는 것이 얼마나 중요한지를 다시 한번 느끼는 계기가 됨`


## Memory Extension

![mem-ext](./images/03p_mem_ext.png)

